/* ARCADEmini USB driver for Linux
 * version 1.2.6 date 2007-11-17
 * part of the ARCADEmini project - http[s]://arcademini.schuermans.info/
 * Copyright (C) 2003-2006 Stefan Schuermans <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#ifndef INC_am_usb
#define INC_am_usb

#include <linux/ioctl.h>
#include <linux/usb.h>

//*** modes of the driver
//defines the behaviour when using normal file operations as read, write, seek
#define am_usb_mode_none -1 //do not change current mode
#define am_usb_mode_null 0 //act as null device
#define am_usb_mode_stream 1 //display stream written into device
#define am_usb_mode_cf 2 //act as compact flash reader / writer

//*** device dependent constants
#define am_usb_out_style_led 0x00 //normal LED matrix
#define am_usb_out_style_latch 0x01 //LATCHed circuit
#define am_usb_out_style_amp 0x02 //AMPlifier circuit

//*** structures for ioctls
//set command pointer
struct t_am_usb_set_cmd_ptr
{
  unsigned long sec_no; //in, out: sector part of command pointer
  unsigned short ofs; //in, out: offset part of command pointer
} st_am_usb_set_cmd_ptr;
//get command pointer
struct t_am_usb_get_cmd_ptr
{
  unsigned long sec_no; //out: sector part of command pointer
  unsigned short ofs; //out: offset part of command pointer
} st_am_usb_get_cmd_ptr;
//load pixel buffer
struct t_am_usb_load_pix_buf
{
  unsigned char pixels[260]; //in: the pixel data read from the pixel buffer
} st_am_usb_load_pix_buf;
//read pixel buffer
struct t_am_usb_read_pix_buf
{
  unsigned char pixels[260]; //out: the pixel data to load into the pixel buffer
} st_am_usb_read_pix_buf;
//load overlay pixel buffer
struct t_am_usb_load_ovp_buf
{
  unsigned char pixels[260]; //in: the pixel data read from the overlay pixel buffer
} st_am_usb_load_ovp_buf;
//read overlay pixel buffer
struct t_am_usb_read_ovp_buf
{
  unsigned char pixels[260]; //out: the pixel data to load into the overlay pixel buffer
} st_am_usb_read_ovp_buf;
//load sector buffer
struct t_am_usb_load_sec_buf
{
  unsigned char data[512]; //in: the data to load into the sector buffer
} st_am_usb_load_sec_buf;
//read sector buffer
struct t_am_usb_read_sec_buf
{
  unsigned char data[512]; //out: the data read from the sector buffer
} st_am_usb_read_sec_buf;
//get CF sector count
struct t_am_usb_get_cf_sec_cnt
{
  unsigned long sec_cnt; //out: number of sectors on compact flash
} st_am_usb_get_cf_sec_cnt;
//write CF sector
struct t_am_usb_write_cf
{
  unsigned long sec_no; //in, out: sector number on compact flash
  unsigned char err; //out: error code (0 for success)
} st_am_usb_write_cf;
//read CF sector
struct t_am_usb_read_cf
{
  unsigned long sec_no; //in, out: sector number on compact flash
  unsigned char err; //out: error code (0 for success)
} st_am_usb_read_cf;
//get internal sector count
struct t_am_usb_get_if_sec_cnt
{
  unsigned long sec_cnt; //out: number of sectors in internal flash
} st_am_usb_get_if_sec_cnt;
//read internal sector
struct t_am_usb_read_if
{
  unsigned long sec_no; //in, out: sector number in internal flash
  unsigned char err; //out: error code (0 for success)
} st_am_usb_read_if;
//get compact flash flag
struct t_am_usb_get_cf_flag
{
  unsigned char cf_flag; //out: != 0 if commands are executed from compact flash
} st_am_usb_get_cf_flag;
//load variables
struct t_am_usb_load_vars
{
  unsigned char vars[256]; //in: the contents of the variables
} st_am_usb_load_vars;
//read variables
struct t_am_usb_read_vars
{
  unsigned char vars[256]; //out: the contents of the variables
} st_am_usb_read_vars;
//load stack
struct t_am_usb_load_stack
{
  unsigned char data[128]; //in: the contents of the stack
} st_am_usb_load_stack;
//read stack
struct t_am_usb_read_stack
{
  unsigned char data[128]; //out: the contents of the stack
} st_am_usb_read_stack;
//set stack pointer
struct t_am_usb_set_stack_ptr
{
  unsigned char stack_ptr; //in, out: the new value for the stack pointer (0-127)
} st_am_usb_set_stack_ptr;
//get stack pointer
struct t_am_usb_get_stack_ptr
{
  unsigned char stack_ptr; //out: the value of the stack pointer (0-127)
} st_am_usb_get_stack_ptr;
//get and set key state
struct t_am_usb_get_set_keys
{
  unsigned char in_pressed, in_pressed_mask; //in: how to modify pressed key state
  unsigned char in_released, in_released_mask; //in: how to modify released key state
  unsigned char state, pressed, released; //out: key state read
} st_am_usb_get_set_keys;
//get and set SNES0 key state
struct t_am_usb_get_set_snes0
{
  unsigned short in_pressed, in_pressed_mask; //in: how to modify pressed key state
  unsigned short in_released, in_released_mask; //in: how to modify released key state
  unsigned short state, pressed, released; //out: key state read
} st_am_usb_get_set_snes0;
//get and set SNES1 key state
struct t_am_usb_get_set_snes1
{
  unsigned short in_pressed, in_pressed_mask; //in: how to modify pressed key state
  unsigned short in_released, in_released_mask; //in: how to modify released key state
  unsigned short state, pressed, released; //out: key state read
} st_am_usb_get_set_snes1;
//set time
struct t_am_usb_set_time
{
  unsigned char sec, min, hour; //in: the time to set the ARCADEmini clock to
} st_am_usb_set_time;
//get time
struct t_am_usb_get_time
{
  unsigned char sec, min, hour; //out: the time of the ARCADEmini clock
} st_am_usb_get_time;
//get firmware version
struct t_am_usb_get_fw_version
{
  unsigned char major, minor, revision; //out: version
  unsigned char year, month, day; //out: release date of the firmware
} st_am_usb_get_fw_version;
//get ID
struct t_am_usb_get_id
{
  unsigned char id[16]; //out: the ID
  unsigned char err; //out: error code (0 for success)
} st_am_usb_get_id;
//set ID
struct t_am_usb_set_id
{
  unsigned char id[16]; //in: the ID
  unsigned char err; //out: error code (0 for success)
} st_am_usb_set_id;
//get output style
struct t_am_usb_get_out_style
{
  unsigned char style; //out: the output style (0x00=LED, 0x01=LATCH)
  unsigned char err; //out: error code (0 for success)
} st_am_usb_get_out_style;
//set output style
struct t_am_usb_set_out_style
{
  unsigned char style; //in: the output style (0x00=LED, 0x01=LATCH)
  unsigned char err; //out: error code (0 for success)
} st_am_usb_set_out_style;
//get output timing
struct t_am_usb_get_out_timing
{
  unsigned char timing[15]; //out: the output timing (0..50)
  unsigned char err; //out: error code (0 for success)
} st_am_usb_get_out_timing;
//set output timing
struct t_am_usb_set_out_timing
{
  unsigned char timing[15]; //in: the output timing (0..50)
  unsigned char err; //out: error code (0 for success)
} st_am_usb_set_out_timing;
//get output greyscale mapping
struct t_am_usb_get_out_greymap
{
  unsigned char greymap[15]; //out: the output greyscale mapping (0..15)
  unsigned char err; //out: error code (0 for success)
} st_am_usb_get_out_greymap;
//set output greyscale mapping
struct t_am_usb_set_out_greymap
{
  unsigned char greymap[15]; //in: the output greyscale mapping (1..15)
  unsigned char err; //out: error code (0 for success)
} st_am_usb_set_out_greymap;
//get version information
struct t_am_usb_get_version
{
  unsigned char major, minor, revision; //out: version
  unsigned short year; //out: release date
  unsigned char month, day;
} st_am_usb_get_version;
//set mode
struct t_am_usb_set_mode
{
  int mode; //in, out: the new mode
  int def_mode; //in, out: the new default mode
} st_am_usb_set_mode;
//get mode
struct t_am_usb_get_mode
{
  int mode; //out: the current mode
  int def_mode; //out: the current default mode
} st_am_usb_get_mode;

//*** ioctls
//halt ARCADEmini
//turns off command processing
#define am_usb_ioctl_no_halt _IO( 0xAC, 0x00 )
//run ARCADEmini
//turns on command processing
#define am_usb_ioctl_no_run _IO( 0xAC, 0x01 )
//step ARCADEmini
//processes a single command
#define am_usb_ioctl_no_step _IO( 0xAC, 0x02 )
//set command pointer
#define am_usb_ioctl_no_set_cmd_ptr _IOWR( 0xAC, 0x08, struct t_am_usb_set_cmd_ptr )
//get command pointer
#define am_usb_ioctl_no_get_cmd_ptr _IOR( 0xAC, 0x09, struct t_am_usb_get_cmd_ptr )
//load pixel buffer
//loads the pixel buffer with 260 bytes of data
#define am_usb_ioctl_no_load_pix_buf _IOW( 0xAC, 0x10, struct t_am_usb_load_pix_buf )
//read pixel buffer
//reads 260 bytes of data from the pixel buffer
#define am_usb_ioctl_no_read_pix_buf _IOR( 0xAC, 0x11, struct t_am_usb_read_pix_buf )
//load overlay pixel buffer
//loads the overlay pixel buffer with 260 bytes of data
#define am_usb_ioctl_no_load_ovp_buf _IOW( 0xAC, 0x12, struct t_am_usb_load_ovp_buf )
//read overlay pixel buffer
//reads 260 bytes of data from the overlay pixel buffer
#define am_usb_ioctl_no_read_ovp_buf _IOR( 0xAC, 0x13, struct t_am_usb_read_ovp_buf )
//swap display buffers
//exchanges the foreground and background pixel buffer
#define am_usb_ioctl_no_swap_bufs _IO( 0xAC, 0x20 )
//swap display buffers and copy image
//exchanges the foreground and background pixel buffer
//then copies the foreground picture into the background buffer
#define am_usb_ioctl_no_swap_bufs_cp _IO( 0xAC, 0x21 )
//load sector buffer
//loads the sector buffer with 512 bytes of data
#define am_usb_ioctl_no_load_sec_buf _IOW( 0xAC, 0x30, struct t_am_usb_load_sec_buf )
//read sector buffer
//reads 512 bytes of data from the sector buffer
#define am_usb_ioctl_no_read_sec_buf _IOR( 0xAC, 0x31, struct t_am_usb_read_sec_buf )
//get CF sector count
//returns the number of sectors on compact flash
#define am_usb_ioctl_no_get_cf_sec_cnt _IOR( 0xAC, 0x40, struct t_am_usb_get_cf_sec_cnt )
//write CF sector
//writes the sector buffer to a sector on compact flash
#define am_usb_ioctl_no_write_cf _IOWR( 0xAC, 0x42, struct t_am_usb_write_cf )
//read CF sector
//reads a sector on compact flash into sector buffer
#define am_usb_ioctl_no_read_cf _IOWR( 0xAC, 0x43, struct t_am_usb_read_cf )
//get internal sector count
//returns the number of sectors in internal flash
#define am_usb_ioctl_no_get_if_sec_cnt _IOR( 0xAC, 0x44, struct t_am_usb_get_if_sec_cnt )
//read internal sector
//reads a sector in internal flash into sector buffer
#define am_usb_ioctl_no_read_if _IOWR( 0xAC, 0x47, struct t_am_usb_read_if )
//get compact flash flag
//returns if commands are executed from compact flash or from internal flash
#define am_usb_ioctl_no_get_cf_flag _IOR( 0xAC, 0x48, struct t_am_usb_get_cf_flag )
//load variables
//loads the variables with 256 bytes of data
#define am_usb_ioctl_no_load_vars _IOW( 0xAC, 0x50, struct t_am_usb_load_vars )
//read variables
//reads 256 bytes of data from the variables
#define am_usb_ioctl_no_read_vars _IOR( 0xAC, 0x51, struct t_am_usb_read_vars )
//load stack
//loads the stack with 128 bytes of data
#define am_usb_ioctl_no_load_stack _IOW( 0xAC, 0x54, struct t_am_usb_load_stack )
//read stack
//reads 128 bytes of data from the stack
#define am_usb_ioctl_no_read_stack _IOR( 0xAC, 0x55, struct t_am_usb_read_stack )
//set stack pointer
#define am_usb_ioctl_no_set_stack_ptr _IOWR( 0xAC, 0x56, struct t_am_usb_set_stack_ptr )
//get stack pointer
#define am_usb_ioctl_no_get_stack_ptr _IOR( 0xAC, 0x57, struct t_am_usb_get_stack_ptr )
//get and set key state
//gets key state and sets key state afterwards
#define am_usb_ioctl_no_get_set_keys _IOWR( 0xAC, 0x60, struct t_am_usb_get_set_keys )
//get and set SNES0 key state
//gets SNES0 key state and sets SNES0 key state afterwards
#define am_usb_ioctl_no_get_set_snes0 _IOWR( 0xAC, 0x62, struct t_am_usb_get_set_snes0 )
//get and set SNES1 key state
//gets SNES1 key state and sets SNES1 key state afterwards
#define am_usb_ioctl_no_get_set_snes1 _IOWR( 0xAC, 0x63, struct t_am_usb_get_set_snes1 )
//set time
//sets the time of the ARCADEmini closk
#define am_usb_ioctl_no_set_time _IOW( 0xAC, 0x70, struct t_am_usb_set_time )
//get time
//gets the time from the ARCADEmini clock
#define am_usb_ioctl_no_get_time _IOW( 0xAC, 0x71, struct t_am_usb_get_time )
//get firmware version
//returns the firmware version of ARCADEmini
#define am_usb_ioctl_no_get_fw_version _IOR( 0xAC, 0x80, struct t_am_usb_get_fw_version )
//get ID
//gets the ID of this ARCADEmini
#define am_usb_ioctl_no_get_id _IOR( 0xAC, 0x84, struct t_am_usb_get_id )
//set ID
//sets the ID of this ARCADEmini
#define am_usb_ioctl_no_set_id _IOWR( 0xAC, 0x85, struct t_am_usb_set_id )
//get output style
//gets the output style
#define am_usb_ioctl_no_get_out_style _IOR( 0xAC, 0x88, struct t_am_usb_get_out_style )
//set output style
//sets the output style
#define am_usb_ioctl_no_set_out_style _IOWR( 0xAC, 0x89, struct t_am_usb_set_out_style )
//get output timing
//gets the output timing
#define am_usb_ioctl_no_get_out_timing _IOR( 0xAC, 0x8A, struct t_am_usb_get_out_timing )
//set output timing
//sets the output timing
#define am_usb_ioctl_no_set_out_timing _IOWR( 0xAC, 0x8B, struct t_am_usb_set_out_timing )
//get output greyscale mapping
//gets the output greyscale mapping
#define am_usb_ioctl_no_get_out_greymap _IOR( 0xAC, 0x8C, struct t_am_usb_get_out_greymap )
//set output greyscale mapping
//sets the output greyscale mapping
#define am_usb_ioctl_no_set_out_greymap _IOWR( 0xAC, 0x8D, struct t_am_usb_set_out_greymap )
//get version information
//returns version information about the driver
#define am_usb_ioctl_no_get_version _IOR( 0xAC, 0xF0, struct t_am_usb_get_version )
//set mode
//sets the driver mode for normal file operations
#define am_usb_ioctl_no_set_mode _IOWR( 0xAC, 0xF2, struct t_am_usb_set_mode )
//get mode
//gets the driver mode for normal file operations
#define am_usb_ioctl_no_get_mode _IOR( 0xAC, 0xF3, struct t_am_usb_get_mode )

#endif //#ifndef INC_am_usb

